@@ -10,11 +10,13 @@ import os
 import uuid
 from collections import OrderedDict
 import logging
+logger = logging.getLogger("sdata")
 import numpy as np
 import pandas as pd
 import shutil
 import copy
 from sdata.metadata import Metadata, Attribute, MetadataSchema, AttributeSchema, extract_name_unit
+from sdata.timestamp import now_utc_str, now_local_str, today_str
 import inspect
 import json
 import hashlib
@@ -22,6 +24,8 @@ import base64
 import requests
 from io import BytesIO, StringIO
 
+class Sdata_Name_Exeption(Exception): pass
+class Sdata_Uuid_Exeption(Exception): pass
 
 if sys.version_info < (3, 6):
     import sha3
@@ -29,7 +33,7 @@ if sys.version_info < (3, 6):
 try:
     import openpyxl
 except:
-    logging.warning("openpyxl is not available -> no xlsx import")
+    logger.warning("openpyxl is not available -> no xlsx import")
 
 def uuid_from_str(name):
     return uuid.uuid3(uuid.NAMESPACE_DNS, name)
@@ -38,6 +42,12 @@ class Data(object):
     """Base sdata object"""
     ATTR_NAMES = []
 
+    SDATA_VERSION = "!sdata_version"
+    SDATA_NAME = "!sdata_name"
+    SDATA_UUID = "!sdata_uuid"
+    SDATA_CTIME = "!sdata_ctime"
+    SDATA_MTIME = "!sdata_mtime"
+
     def __init__(self, **kwargs):
         """create Data object
 
@@ -62,12 +72,30 @@ class Data(object):
         self._prefix = None
         # ToDo: add getter and setter for metadata
         self.metadata = kwargs.get("metadata") or Metadata()
-        _uuid = kwargs.get("uuid") or ""
-        _name = kwargs.get("name") or "N.N."
-        self.metadata.add("name", _name)
-        self.metadata.add("uuid", _uuid)
 
-        self.uuid = kwargs.get("uuid") or uuid.uuid4()
+        # set default sdata attributes
+        self.metadata.add(self.SDATA_VERSION, __version__, dtype="str")
+        self.metadata.add(self.SDATA_NAME, "", dtype="str")
+        self.metadata.add(self.SDATA_UUID, "", dtype="str")
+        self.metadata.add(self.SDATA_CTIME, now_utc_str(), dtype="str")
+        self.metadata.add(self.SDATA_MTIME, now_utc_str(), dtype="str")
+
+        # auto correct
+        if kwargs.get("auto_correct") is None or kwargs.get("auto_correct") is True:
+            self.auto_correct = True
+        else:
+            self.auto_correct = False
+
+        logger.debug("sdata: set auto_correct={}".format(self.auto_correct))
+
+        try:
+            self.uuid = kwargs.get("uuid") or uuid.uuid4() # store given uuid str or generate a new uuid
+        except Sdata_Uuid_Exeption as exp:
+            if self.auto_correct is True:
+                logger.warning("got invald uuid -> generate a new uuid")
+                self.uuid = uuid.uuid4()
+            else:
+                raise
         self.name = kwargs.get("name") or "N.N."
         self.prefix = kwargs.get("prefix") or ""
         self._gen_default_attributes(kwargs.get("default_attributes") or self.ATTR_NAMES)
@@ -77,6 +105,13 @@ class Data(object):
         self._description = ""
         self.description = kwargs.get("description", "")
 
+    def update_mtime(self):
+        """update modification time
+
+        :return:
+        """
+        self.metadata.add(self.SDATA_MTIME, now_utc_str())
+
     @property
     def sha3_256(self):
         """Return a new SHA3 hash object with a hashbit length of 32 bytes.
@@ -127,7 +162,7 @@ class Data(object):
         :return: hash
         """
         if not (hasattr(hashobject, "update") and hasattr(hashobject, "hexdigest")):
-            logging.error("Data.update_hash: given hashfunction is invalid")
+            logger.error("Data.update_hash: given hashfunction is invalid")
             raise Exception("Data.update_hash: given hashfunction is invalid")
 
         metadatastr = self.metadata.to_json().encode(errors="replace")
@@ -178,35 +213,38 @@ class Data(object):
             self.metadata.set_attr(name=attr_name, value=value, dtype=dtype, description=description)
 
     def _get_uuid(self):
-        return self.metadata.get("uuid").value
+        return self.metadata.get(self.SDATA_UUID).value
         # return self._uuid
 
     def _set_uuid(self, value):
         if isinstance(value, str):
             try:
-                self._uuid = uuid.UUID(value).hex
+                uuid.UUID(value)
+                self.metadata.set_attr(self.SDATA_UUID, uuid.UUID(value).hex)
             except ValueError as exp:
-                logging.warning("data.uuid: %s" % exp)
+                logger.warning("data.uuid: %s" % exp)
+                raise Sdata_Uuid_Exeption("got invalid uuid str '{}'".format(str(value)))
         elif isinstance(value, uuid.UUID):
-            self.metadata.set_attr("uuid", value.hex)
+            self.metadata.set_attr(self.SDATA_UUID, value.hex)
         else:
-            logging.error("Data.uuid: invalid uuid '{}'".format(value))
+            logger.error("Data.uuid: invalid uuid '{}'".format(value))
+            raise Exception("Data.uuid: invalid uuid '{}'".format(value))
 
     uuid = property(fget=_get_uuid, fset=_set_uuid, doc="uuid of the object")
 
     def _get_name(self):
         # return self._name
-        return self.metadata.get("name").value
+        return self.metadata.get(self.SDATA_NAME).value
 
     def _set_name(self, value):
         if isinstance(value, str):
             try:
-                self.metadata.set_attr("name", str(value)[:256])
+                self.metadata.set_attr(self.SDATA_NAME, str(value)[:256])
             except ValueError as exp:
-                logging.warning("data.name: %s" % exp)
+                logger.warning("data.name: %s" % exp)
         else:
             # self._name = str(value)[:256]
-            self.metadata.set_attr("name", str(value)[:256])
+            self.metadata.set_attr(self.SDATA_NAME, str(value)[:256])
 
     name = property(fget=_get_name, fset=_set_name, doc="name of the object")
 
@@ -218,7 +256,7 @@ class Data(object):
             try:
                 self._description = str(value)
             except ValueError as exp:
-                logging.warning("data.name: %s" % exp)
+                logger.warning("data.name: %s" % exp)
         else:
             self._description = str(value)
 
@@ -247,7 +285,7 @@ class Data(object):
             try:
                 self._prefix = value[:256]
             except ValueError as exp:
-                logging.warning("data.prefix: %s" % exp)
+                logger.warning("data.prefix: %s" % exp)
         else:
             self._prefix = str(value)[:256]
 
@@ -265,14 +303,20 @@ class Data(object):
     df = table
 
     def to_folder(self, path, dtype="csv"):
-        """export data to folder"""
+        """export data to folder
+
+        :param path:
+        :param dtype:
+        :return:
+        """
+
         if dtype not in ["csv", "xlsx"]:
             dtype = "xlsx"
         if not os.path.exists(path):
             try:
                 os.makedirs(path)
             except OSError as exp:
-                logging.error(exp)
+                logger.error(exp)
         else:
             self.clear_folder(path)
 
@@ -283,7 +327,7 @@ class Data(object):
 
         if dtype == "csv":
             metadata_filepath = os.path.join(path, "metadata.csv")
-            logging.debug("export meta csv '{}'".format(metadata_filepath))
+            logger.debug("export meta csv '{}'".format(metadata_filepath))
             self.metadata.to_csv(metadata_filepath)
 
             # table export
@@ -303,12 +347,16 @@ class Data(object):
 
     @classmethod
     def from_folder(cls, path):
-        """:returns: sdata object instance"""
+        """sdata object instance
+
+        :param path:
+        :return:
+        """
         # data = Data.from_folder(path)
 
         data = cls()
         if not os.path.exists(path):
-            logging.error("from_folder error: path '{}' not exists.".format(path))
+            logger.error("from_folder error: path '{}' not exists.".format(path))
             return data
 
         data.metadata = data._load_metadata(path)
@@ -316,7 +364,7 @@ class Data(object):
             data.uuid = data.metadata.get_attr("uuid").value
             data.name = data.metadata.get_attr("name").value
         except Exception as exp:
-            logging.error("Data.from_folder: {}".format(data.metadata.to_dict()))
+            logger.error("Data.from_folder: {}".format(data.metadata.to_dict()))
             raise
 
         # table import
@@ -364,7 +412,7 @@ class Data(object):
         for subfolder in valid_subfolders:
             try:
                 subfolder = os.path.join(path, subfolder)
-                logging.debug("clear_folder: rm {}".format(subfolder))
+                logger.debug("clear_folder: rm {}".format(subfolder))
                 shutil.rmtree(subfolder)
             except OSError as exp:
                 raise
@@ -391,10 +439,10 @@ class Data(object):
             sdataclassname = classattr.value
             sdatacls = SDATACLS.get(sdataclassname)
             if sdataclassname not in SDATACLS:
-                logging.warning("unsupported cls '{}'".format(sdataclassname))
+                logger.warning("unsupported cls '{}'".format(sdataclassname))
                 sdatacls = Data
         else:
-            logging.warn("cls not defined '{}'".format(metadata))
+            logger.warning("cls not defined '{}'".format(metadata))
             sdatacls = None
         return sdatacls
 
@@ -458,11 +506,11 @@ class Data(object):
         if isinstance(data, Data):
             names = [dat.name.lower() for uid, dat in self.group.items()]
             if data.name.lower() in names:
-                logging.error("{}: name '{}' aready exists".format(data.__class__.__name__, data.name))
+                logger.error("{}: name '{}' aready exists".format(data.__class__.__name__, data.name))
                 return
             self.group[data.uuid] = data
         else:
-            logging.warning("ignore data %s (wrong type!)" % data)
+            logger.warning("ignore data %s (wrong type!)" % data)
 
     def get_data_by_uuid(self, uid):
         """get data by uuid"""
@@ -632,7 +680,7 @@ class Data(object):
                 if "table" in sheetnames:
                     tt.table = pd.read_excel(filepath, sheet_name="table", index_col='index')
                 else:
-                    logging.info("no table data in '{}'".format(filepath))
+                    logger.info("no table data in '{}'".format(filepath))
                 dfm = pd.read_excel(filepath, sheet_name="metadata")
                 dfm = dfm.set_index(dfm.name.values)
                 # dfm["value"] = dfm["value"].replace(np.nan, None)
@@ -651,7 +699,7 @@ class Data(object):
                             cells.append("")
                     tt.description = "\n".join(cells)
                 else:
-                    logging.info("no description in '{}'".format(filepath))
+                    logger.info("no description in '{}'".format(filepath))
 
                 return tt
             else:
@@ -694,31 +742,31 @@ class Data(object):
             with open(filepath, "r") as fh:
                 d = json.load(fh)
         elif s is None and filepath is None:
-            logging.error("data.from_json: no json data available")
+            logger.error("data.from_json: no json data available")
             return
         elif s is not None and filepath is None:
             d = json.loads(s)
         else:
-            logging.error("data.from_json: unexpected error")
+            logger.error("data.from_json: unexpected error")
             d = None
 
         if d:
             if "metadata" in d.keys():
                 data.metadata = data.metadata.from_dict(d["metadata"])
             else:
-                logging.error("Data.from_json: table not available")
+                logger.error("Data.from_json: table not available")
 
             if "table" in d.keys():
                 data.table = pd.DataFrame.from_dict(d["table"])
                 # data.table = pd.read_json(json.dumps(d["table"]))
                 # data.table = pd.read_json(d["table"])
             else:
-                logging.error("Data.from_json: metadata not available")
+                logger.error("Data.from_json: metadata not available")
 
             if "description" in d.keys():
                 data.description = d["description"]
             else:
-                logging.error("Data.from_json: description not available")
+                logger.error("Data.from_json: description not available")
 
         return data
 
@@ -779,7 +827,7 @@ class Data(object):
             pd.read_csv(sio, sep=";", comment="#")
             sio.seek(0)
         else:
-            logging.error("data.from_csv: no csv data available")
+            logger.error("data.from_csv: no csv data available")
             return
 
         for line in sio:
@@ -836,16 +884,13 @@ class Data(object):
                 if add_table_metadata:
                     old_attr = self.metadata.get(old_colname)
                     if old_attr:
-                        logging.info("skip: {}".format(old_attr))
+                        logger.info("skip: {}".format(old_attr))
                         self.metadata.relabel(old_colname, name)
                     else:
                         self.metadata.add(name=name, description=old_colname, unit=unit, dtype="float")
             self.table.rename(columns=mapper, inplace=True)
 
 
-
-
-
 class Schema(Data):
     """Base sdata object"""
     ATTR_NAMES = []
